import torch

def batch_roll(X, shifts_x,shifts_y):
    """
        Like torch.roll, but supports a batch of shifts. Only applicable for 2D tensors.
        
        X : (H,W) tensor
        shifts_x : (B,) tensor
        shifts_y : (B,) tensor

        Returns : (B,H,W) tensor, shifted accordingly
    """
    assert shifts_x.shape==shifts_y.shape
    B = shifts_x.shape[0]
    H,W = X.shape


    shifts_x = shifts_x[:,None]
    shifts_y = shifts_y[:,None]

    new_x = (torch.arange(H)[None,:] - shifts_x)%H # (B,H)
    new_y = (torch.arange(W)[None,:] - shifts_y)%W # (B,W)
    
    return X[new_x[:,:,None],new_y[:,None,:]]

if __name__=='__main__':
    X = torch.arange(9).reshape(3,3)
    shifts_x = torch.arange(-1,2)
    shifts_y = torch.arange(-1,2,1)

    print(shifts_x, shifts_y)

    rolled= batched_roll(X,shifts_x,shifts_y)
    print(f'rolled shape {rolled.shape}')
    print(f'rolled result : \n{rolled}')



import torch,sys,pathlib, torch.nn as nn, os
sys.path.append(pathlib.Path(__file__).parent.parent.as_posix())

curpath = pathlib.Path(__file__).parent
from src.torchenhanced.modules import DevModule, ConfigModule


def test_device_initialization():
    module = DevModule(device='cpu')
    assert str(module.device) == 'cpu', 'Problem in DevModule device initialization'

    if torch.cuda.is_available():
        module_cuda = DevModule(device='cuda')
        assert str(module_cuda.device).startswith('cuda'), 'Problem in DevModule device initialization'

def test_paranum_empty_module():
    module = DevModule()
    assert module.paranum==0, 'Problem in DevModule paranum'


def make_test_configmodule():
    class TestConfigModule(ConfigModule):
        def __init__(self, hidden = 28*28, out =10):
            config = {'hidden':hidden, 'out':out}

            super().__init__(config)

            self.layer = nn.Linear(hidden, out)
        
        def forward(self, x):
            return self.layer(x)

    testmodule = TestConfigModule()
    
    return testmodule

def test_configmodule_config():
    module = make_test_configmodule()
    config = module.config
    assert config['hidden']==28*28, 'Problem in ConfigModule config'
    assert config['out']==10, 'Problem in ConfigModule config'

def test_configmodule_weights():
    module = make_test_configmodule()

    module.save_weights(os.path.join(curpath,'test_torchenhanced/test_weights.pt'))

    module.load_weights(os.path.join(curpath,'test_torchenhanced/test_weights.pt'))

    if( torch.cuda.is_available() ):
        module.to('cuda')
    
    module.load_weights(os.path.join(curpath,'test_torchenhanced/test_weights.pt'))

    assert module.paranum>0, 'Problem in ConfigModule load_weights'

test_configmodule_weights()